<template>
  <div class="mod-config">
    <basic-container>
      <ttable
        :table="table"
        @handleSelectionChange="handleSelectionChange"
        :uploadInfo="uploadInfo"
        :prelang="prelang"
        :options="options"
        :ajaxFun="ajaxFun"
        ref="unqualifiedTable"
      >
        <template #toolbar>
          <el-button
            type="primary"
            v-if="permissions.quality_unqualifiedprocess_isused"
            :loading="isusedSynsLoading"
            @click="batchUpdateResultIsUsed"
            >允许使用并移库</el-button
          >
        </template>
      </ttable>
      <!-- 弹窗, 处理新增 / 修改 -->
      <unqualifiedProcessForm
        v-if="addOrUpdateVisible"
        ref="addOrUpdate"
        @refreshDataList="getData"
      />
      <!-- 弹窗, 审批 -->
      <examineApproveForm
        v-if="addexamineVisible"
        ref="addexamine"
        @refreshDataList="getData"
      />
      <!-- 弹窗, 执行 -->
      <tableForm
        v-if="addtableVisible"
        ref="addtable"
        @refreshDataList="getData"
      />
      <!-- 损失登记弹窗 -->
      <registerLossForm
        :registerLossList="multipleSelection"
        :currshowlist.sync="showRegisterLoss"
        @refreshDataList="getData"
      />
      <!-- 零件 -->
      <part-dialog
        :currshowlist.sync="showPart"
        @listenToPartEvent="selectPart"
      />
    </basic-container>
  </div>
</template>

<script>
import {
  fetchListUnqualifiedProcess,
  delUnqualifiedProcess,
  launchOaUnQualified,
  rfOa,
  partDegrade
} from '@/api/quality/unqualifiedprocess'
import { batchUpdateResultBySystemNo } from '@/api/quality/result'
import unqualifiedProcessForm from './unqualifiedprocessform'
import examineApproveForm from './examineApproveform'
import registerLossForm from './registerlossform'
import tableForm from './table-form'
import PartDialog from '@/views/common/part.vue'
import { mapGetters } from 'vuex'
import ttable from '@/views/common/ztt-table.vue'
import { remote } from '@/api/admin/dict'

export default {
  data() {
    return {
      ajaxFun: fetchListUnqualifiedProcess,
      addOrUpdateVisible: false,
      addexamineVisible: false,
      addtableVisible: false,
      currRow: null,
      multipleSelection: [],
      uploadInfo: {
        // 是否展示上传EXCEL以及对应的url
        isShow: false,
        url: ''
      },
      prelang: 'unqualifiedTable',
      options: {
        height: 300, // 默认高度-为了表头固定
        stripe: true, // 是否为斑马纹 table
        highlightCurrentRow: false, // 是否要高亮当前行
        border: true, // 是否有纵向边框
        lazy: false, // 是否需要懒加载
        fit: true, // 列的宽度是否自撑开
        multiSelect: true, //
        seqNo: true,
        isRefresh: true, // 是否显示刷新按钮
        isShowHide: true, // 是否显示显影按钮
        isSearch: false, // 高级查询按钮
        defaultOrderBy: { column: 'processNo', direction: 'desc' }
      },
      table: {
        total: 0,
        currentPage: 1,
        pageSize: 20,
        data: [],
        // 标题
        column: [
          {
            width: '100',
            minWidth: '100',
            prop: 'processNo',
            label: '编号',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text',
            render: { fun: this.addOrUpdateHandle }
          },
          {
            width: '100',
            minWidth: '100',
            prop: 'mainProcessNo',
            label: '处理编号',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            width: '100',
            minWidth: '100',
            prop: 'reason',
            label: '原因',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '120',
            prop: 'causeType',
            label: '原因类型',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'select',
            formatter: this.formatCauseType,
            optList: () => {
              return this.causeTypeOptions
            }
          },
          {
            width: '100',
            minWidth: '100',
            prop: 'reportType',
            label: '汇报类型',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'select',
            formatter: this.formatReportType,
            optList: () => {
              return this.reportTypeOptions
            }
          },
          {
            minWidth: '120',
            prop: 'processMode',
            label: '申请处理方式',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'select',
            formatter: this.formatProcessMode,
            optList: () => {
              return this.processModeOptions
            }
          },
          {
            minWidth: '120',
            prop: 'type',
            label: '种类',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'select',
            formatter: this.formatType,
            optList: () => {
              return this.typeOptions
            }
          },
          {
            minWidth: '120',
            prop: 'unqualifiedType',
            label: '不合格类别',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'select',
            formatter: this.formatUnqualifiedType,
            optList: () => {
              return this.unqualifiedTypeOptions
            }
          },
          {
            width: '100',
            minWidth: '100',
            prop: 'unqualifiedDesc',
            label: '不合格处理描述',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          // 状态
          {
            minWidth: '120',
            prop: 'state',
            label: '状态',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'select',
            optList: () => {
              return this.getStateOptionList()
            },
            formatter: (row, column, cellValue) => {
              var formatVal
              if (cellValue == '01draft') {
                formatVal = '待审核'
              } else if (cellValue == '02pending') {
                formatVal = 'OA审核中'
              } else if (cellValue == '03accepted') {
                formatVal = '通过'
              } else if (cellValue == '04reject') {
                formatVal = '退回'
              } else {
                formatVal = '已执行'
              }
              return formatVal
            }
          },
          {
            width: '100',
            minWidth: '100',
            prop: 'systemNo',
            label: '系统编号',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            width: '100',
            minWidth: '100',
            prop: 'lotBatchNo',
            label: '零件批号',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            width: '100',
            minWidth: '100',
            prop: 'partNo',
            label: '零件编号',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            width: '100',
            minWidth: '100',
            prop: 'partDesc',
            label: '零件名称',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            width: '100',
            minWidth: '100',
            prop: 'unqualifiedArrived',
            label: '不合格数量',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            width: '100',
            minWidth: '100',
            prop: 'workstationName',
            label: '工作站名称',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            width: '100',
            minWidth: '100',
            prop: 'workstationNo',
            label: '工作站编号',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            width: '100',
            minWidth: '100',
            prop: 'productionUser',
            label: '生产人员',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            width: '100',
            minWidth: '100',
            prop: 'productionTime',
            label: '生产日期',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'datetimerange'
          },
          {
            width: '100',
            minWidth: '100',
            prop: 'testUser',
            label: '检验人员',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            width: '100',
            minWidth: '100',
            prop: 'operationName',
            label: '工序名称',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            width: '100',
            minWidth: '100',
            prop: 'workShop',
            label: '车间',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            width: '80',
            minWidth: '80',
            prop: 'isQualified',
            label: '检测合格',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'select',
            formatter: this.formatBoolean,
            optList: () => {
              return this.booleanOptions
            }
          },
          {
            width: '80',
            minWidth: '80',
            prop: 'isUsed',
            label: '允许使用',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'select',
            formatter: this.formatBoolean,
            optList: () => {
              return this.booleanOptions
            }
          },
          {
            width: '100',
            minWidth: '100',
            prop: 'remark',
            label: '备注',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            width: '100',
            minWidth: '100',
            prop: 'examineIdea',
            label: '评审意见',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            width: '100',
            minWidth: '100',
            prop: 'applyNo',
            label: '检测申请编号',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            width: '100',
            minWidth: '100',
            prop: 'reportNo',
            label: '检测汇报编号',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            width: '100',
            minWidth: '100',
            prop: 'lossRegistration',
            label: '质量损失',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            width: '100',
            minWidth: '100',
            prop: 'examiner',
            label: '审批人',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            width: '140',
            minWidth: '140',
            prop: 'examineTime',
            label: '审核时间',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'datetimerange'
          },
          {
            width: '100',
            minWidth: '100',
            prop: 'handler',
            label: '执行人',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            width: '140',
            minWidth: '140',
            prop: 'handleTime',
            label: '执行时间',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'datetimerange'
          },
          {
            width: '100',
            minWidth: '100',
            prop: 'degradePartNo',
            label: '降级后零件号',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            width: '140',
            minWidth: '140',
            prop: 'degradePartName',
            label: '降级后零件名称',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            width: '100',
            minWidth: '100',
            prop: 'executeSn',
            label: '处理后缆号',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            width: '100',
            minWidth: '100',
            prop: 'createUser',
            label: '创建人',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            width: '140',
            minWidth: '140',
            prop: 'createTime',
            label: '创建时间',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'datetimerange'
          }
        ],
        toolbar: [
          {
            text: '新增处理',
            type: 'primary',
            fun: this.addOrUpdateHandle
          },
          {
            text: '损失登记',
            type: 'primary',
            fun: this.openRegisterLoss
          },
          {
            text: '推送OA',
            type: 'primary',
            loading: false,
            fun: this.launchRfOaUnQualified
          },
          {
            text: '执行',
            type: 'primary',
            loading: false,
            fun: this.implementIfsUnQualified
          },
          {
            text: '审批',
            type: 'primary',
            loading: false,
            fun: this.examineApprove
          },
          {
            text: '零件降级功能',
            type: 'primary',
            loading: false,
            fun: this.partDegradation
          }
        ],
        operator: [
          {
            text: '删除',
            type: 'text',
            size: 'small',
            fun: this.deleteHandle
          }
        ],
        operatorConfig: {
          fixed: 'right',
          label: '操作',
          width: 60,
          minWidth: 60
        }
      },
      booleanOptions: [
        { value: true, label: '是' },
        { value: false, label: '否' }
      ],
      stateOptionList: [
        {
          value: '01draft',
          label: '待审核'
        },
        {
          value: '02pending',
          label: 'OA审核中'
        },
        {
          value: '03accepted',
          label: '通过'
        },
        {
          value: '04reject',
          label: '退回'
        },
        {
          value: '05executed',
          label: '已执行'
        }
      ],
      showRegisterLoss: false,
      showPart: false,
      causeTypeOptions: [],
      processModeOptions: [],
      typeOptions: [],
      unqualifiedTypeOptions: [],
      isusedSynsLoading: false,
      reportTypeOptions: []
    }
  },
  components: {
    unqualifiedProcessForm,
    registerLossForm,
    ttable,
    examineApproveForm,
    tableForm,
    PartDialog
  },
  computed: {
    ...mapGetters(['permissions', 'userInfo'])
  },
  created() {
    this.getCauseType()
    this.getProcessMode()
    this.getType()
    this.getUnqualifiedType()
    this.getReportTypeDict()
  },
  methods: {
    partDegradation() {
      if (this.multipleSelection.length > 0) {
        if (this.multipleSelection.length == 1) {
          this.showPart = true
        } else {
          this.$message.error('请选择一条数据进行零件降级功能！')
        }
      } else {
        this.$message.error('请选择需要零件降级的数据！')
      }
    },
    selectPart(part) {
      console.log(part)
      if (part) {
        partDegrade(this.multipleSelection[0].id, part.id).then((res) => {
          console.log(res)
          this.$message.success('零件降级成功！')
        })
      }
    },
    formatCauseType(row, column, cellValue) {
      this.causeTypeOptions.forEach((obj) => {
        if (obj.value === cellValue) {
          cellValue = obj.label
        }
      })
      return cellValue
    },
    getCauseType() {
      remote('unqualified_reason_type').then((response) => {
        if (response.data.code === 0) {
          this.causeTypeOptions = response.data.data
        }
      })
    },
    formatProcessMode(row, column, cellValue) {
      this.processModeOptions.forEach((obj) => {
        if (obj.value === cellValue) {
          cellValue = obj.label
        }
      })
      return cellValue
    },
    getProcessMode() {
      remote('unqualified_process_mode').then((response) => {
        if (response.data.code === 0) {
          this.processModeOptions = response.data.data
        }
      })
    },
    formatType(row, column, cellValue) {
      this.typeOptions.forEach((obj) => {
        if (obj.value === cellValue) {
          cellValue = obj.label
        }
      })
      return cellValue
    },
    getType() {
      remote('unqualified_handle_type').then((response) => {
        if (response.data.code === 0) {
          this.typeOptions = response.data.data
        }
      })
    },
    formatUnqualifiedType(row, column, cellValue) {
      this.unqualifiedTypeOptions.forEach((obj) => {
        if (obj.value === cellValue) {
          cellValue = obj.label
        }
      })
      return cellValue
    },
    getUnqualifiedType() {
      remote('unqualified_handle_category').then((response) => {
        if (response.data.code === 0) {
          this.unqualifiedTypeOptions = response.data.data
        }
      })
    },
    // 获取数据列表
    getData() {
      this.$refs.unqualifiedTable.getDataList()
    },
    // 新增 / 修改
    addOrUpdateHandle(row) {
      this.addOrUpdateVisible = true
      this.$nextTick(() => {
        this.$refs.addOrUpdate.init(row == null ? null : row.id)
      })
    },
    examineApprove() {
      if (this.multipleSelection.length > 0) {
        if (this.multipleSelection.length == 1) {
          this.addexamineVisible = true
          this.$nextTick(() => {
            this.$refs.addexamine.init(this.multipleSelection)
          })
        } else {
          this.$message.error('请选择一条数据进行审批！')
        }
      } else {
        this.$message.error('请选择需要审批的数据！')
      }
    },
    getStateOptionList() {
      return this.stateOptionList
    },
    // 删除
    deleteHandle(row) {
      if (row.state === '01draft' || row.state === '02pending') {
        this.$confirm('是否确认删除处理编号为：' + row.processNo, '提示', {
          confirmButtonText: '确定',
          cancelButtonText: '取消',
          closeOnClickModal: false,
          type: 'warning'
        })
          .then(function() {
            return delUnqualifiedProcess(row.id)
          })
          .then((data) => {
            this.$message.success('删除成功')
            this.getData()
          })
      } else {
        this.$message.warning('该状态下，不可进行删除')
      }
    },
    handleCurrentChange(val) {
      this.currRow = val
    },
    // boolean列中文格式化
    formatBoolean(row, column, cellValue) {
      if (cellValue == true) {
        return '是'
      }
      if (cellValue == false) {
        return '否'
      } else {
        return ''
      }
    },
    // 允许使用并移库
    batchUpdateResultIsUsed() {
      if (this.multipleSelection.length > 0) {
        this.$confirm('是否标记允许使用并移库', '提示', {
          confirmButtonText: '确定',
          cancelButtonText: '取消',
          closeOnClickModal: false,
          type: 'warning'
        })
          .then(() => {
            this.isusedSynsLoading = true
            const resultList = []
            /* this.multipleSelection.forEach((e) => {
              resultList.push(
                Object.assign({
                  isUsed: true,
                  systemNo: e.systemNo,
                  applyPartId: e.applyPartId
                })
              )
            }) */
            resultList.push(
              Object.assign({
                isUsed: true,
                systemNo: this.multipleSelection[0].systemNo,
                applyPartId: this.multipleSelection[0].applyPartId,
                id: this.multipleSelection[0].resultId
              })
            )

            batchUpdateResultBySystemNo(resultList)
              .then((data) => {
                this.isusedSynsLoading = false
                this.$message.success('标记成功')
                this.getData()
              })
              .catch(() => {
                this.isusedSynsLoading = false
              })
          })
          .catch(() => {})
      } else {
        this.$message.warning('请选择不合格零件')
      }
    },
    // for射频
    launchRfOaUnQualified() {
      if (this.multipleSelection.length > 0) {
        if (this.multipleSelection.length === 1) {
          const currSelect = this.multipleSelection[0]
          if (
            currSelect.state === '01draft' ||
            currSelect.state === '02pending'
          ) {
            this.table.toolbar[2].loading = true

            const idParam = { id: currSelect.id }
            rfOa(idParam)
              .then((response) => {
                const resData = response.data
                if (resData.code === 0) {
                  this.$message.success('推送OA成功')
                  this.getData()
                } else {
                  this.$message.error('推送OA失败')
                }
                this.table.toolbar[2].loading = false
              })
              .catch(() => {
                this.table.toolbar[2].loading = false
              })
          } else {
            this.$message.warning('该状态下，无法推送OA')
          }
        } else {
          this.$message.warning('只能选择一个不合格零件')
        }
      } else {
        this.$message.warning('请选择不合格零件')
      }
    },
    launchOaUnQualified() {
      if (this.multipleSelection.length > 0) {
        this.table.toolbar[3].loading = true
        launchOaUnQualified(this.multipleSelection[0].id)
          .then((response) => {
            const resData = response.data
            if (resData.code === 0) {
              this.$message.success('推送OA成功')
              this.getData()
            } else {
              this.$message.error('推送OA失败')
            }
            this.table.toolbar[3].loading = false
          })
          .catch(() => {
            this.table.toolbar[3].loading = false
          })
      } else {
        this.$message.warning('请选择不合格零件')
      }
    },
    // 执行
    implementIfsUnQualified() {
      if (this.multipleSelection.length > 0) {
        this.addtableVisible = true
        this.$nextTick(() => {
          this.$refs.addtable.init(this.multipleSelection)
        })
      } else {
        this.$message.error('请选择不合格零件')
      }
    },
    // 打开损失登记
    openRegisterLoss() {
      if (this.permissions.quality_loss_registration) {
        if (this.multipleSelection.length > 0) {
          this.showRegisterLoss = true
        } else {
          this.$message.error('请选择不合格零件')
        }
      } else {
        this.$message.error('没有权限进行损失登记操作！')
      }
    },
    handleSelectionChange(val) {
      this.multipleSelection = val
    },
    // 汇报类型中文格式化
    formatReportType(row, column, cellValue) {
      this.reportTypeOptions.forEach((obj) => {
        if (obj.value === cellValue) {
          cellValue = obj.label
        }
      })
      return cellValue
    },
    getReportTypeDict() {
      remote('apply_report_type').then((response) => {
        if (response.data.code === 0) {
          this.reportTypeOptions = response.data.data
        } else {
          this.reportTypeOptions = []
        }
      })
    }
  }
}
</script>
